@@ -2,7 +2,7 @@ __author__ = 'Altertech, https://www.altertech.com/'
 __copyright__ = 'Copyright (C) 2019 Altertech'
 __license__ = 'MIT'
 
-__version__ = '0.4.30'
+__version__ = '0.4.36'
 
 from sqlalchemy.exc import IntegrityError
 from cachetools import TTLCache
@@ -260,7 +260,7 @@ def get_version():
 
 
 @core_method
-def exec_query(q, _grafana=False):
+def exec_query(q, _time_ms=False):
     """
     Execute FinacQL query statement
 
@@ -277,7 +277,7 @@ def exec_query(q, _grafana=False):
     * get_version
     * asset_list
     * asset_list_rates
-    * asset_rate
+    * asset_rate^
     * asset_rate_range^
     * account_info
     * account_statement
@@ -326,14 +326,19 @@ def exec_query(q, _grafana=False):
         for row in asset_list(*args, **kwargs):
             yield row
     elif fn == 'asset_list_rates':
-        for row in asset_list_rates(*args, _grafana=_grafana, **kwargs):
+        for row in asset_list_rates(*args, _time_ms=_time_ms, **kwargs):
             yield row
     elif fn == 'asset_rate':
         result = asset_rate(*args,
-                            _grafana=_grafana,
+                            _time_ms=_time_ms,
                             return_pair=True,
                             **kwargs)
-        yield {'pair': result[0], 'rate': result[1]}
+        yield {
+            override_dc_name: result[1]
+        } if override_dc_name else {
+            'pair': result[0],
+            'rate': result[1]
+        }
     elif fn == 'account_info':
         result = account_info(*args, **kwargs)
         if isinstance(result, dict):
@@ -342,25 +347,25 @@ def exec_query(q, _grafana=False):
             for row in result:
                 yield row
     elif fn == 'account_statement':
-        for row in account_statement(*args, _grafana=_grafana, **kwargs):
+        for row in account_statement(*args, _time_ms=_time_ms, **kwargs):
             yield row
     elif fn == 'account_list':
-        for row in account_list(*args, _grafana=_grafana, **kwargs):
+        for row in account_list(*args, _time_ms=_time_ms, **kwargs):
             yield row
     elif fn == 'account_balance':
         yield {
             override_dc_name if override_dc_name else 'balance':
-                account_balance(*args, _grafana=_grafana, **kwargs)
+                account_balance(*args, _time_ms=_time_ms, **kwargs)
         }
     elif fn == 'account_balance_range':
-        times, data = account_balance_range(*args, _grafana=_grafana, **kwargs)
+        times, data = account_balance_range(*args, _time_ms=_time_ms, **kwargs)
         for t, d in zip(times, data):
             yield {
                 'date': t,
                 override_dc_name if override_dc_name else 'balance': d
             }
     elif fn == 'asset_rate_range':
-        times, data = asset_rate_range(*args, _grafana=_grafana, **kwargs)
+        times, data = asset_rate_range(*args, _time_ms=_time_ms, **kwargs)
         for t, d in zip(times, data):
             yield {
                 'date': t,
@@ -679,7 +684,7 @@ def asset_list_rates(asset=None,
                      start=None,
                      end=None,
                      datefmt=False,
-                     _grafana=False):
+                     _time_ms=False):
     """
     List asset rates
 
@@ -688,19 +693,15 @@ def asset_list_rates(asset=None,
     If asset is not specified, "end" is used as date to get rates for all
     assets
     """
-    if _grafana:
-        if isinstance(start, int):
-            start = start / 1000
-        if isinstance(end, int):
-            end = end / 1000
     if asset:
         cond = ''
         asset = _safe_format(asset.upper())
         if start:
-            dts = parse_date(start, return_timestamp=False)
+            dts = parse_date(start, return_timestamp=False, ms=_time_ms)
             cond += (' and ' if cond else '') + 'd >= \'{}\''.format(dts)
-        dte = parse_date(end, return_timestamp=False) if end else parse_date(
-            return_timestamp=False)
+        dte = parse_date(end, return_timestamp=False,
+                         ms=_time_ms) if end else parse_date(
+                             return_timestamp=False)
         cond += (' and ' if cond else '') + 'd <= \'{}\''.format(dte)
         if asset.find('/') != -1:
             asset_from, asset_to = _safe_format(asset.split('/'))
@@ -722,8 +723,9 @@ def asset_list_rates(asset=None,
                     where {cond} order by d
         """.format(cond=cond)))
     else:
-        d = parse_date(end, return_timestamp=False) if end else parse_date(
-            return_timestamp=False)
+        d = parse_date(end, return_timestamp=False,
+                       ms=_time_ms) if end else parse_date(
+                           return_timestamp=False)
         r = get_db().execute(sql("""
             select
                 a1.code as asset_from,
@@ -861,7 +863,7 @@ def _parse_asset_pair(asset_from, asset_to):
 def asset_rate(asset_from=None,
                asset_to=None,
                date=None,
-               _grafana=False,
+               _time_ms=False,
                asset=None,
                return_pair=False):
     if asset:
@@ -870,11 +872,11 @@ def asset_rate(asset_from=None,
     result = _asset_rate_lookup(asset_from,
                                 asset_to,
                                 date=date,
-                                _grafana=_grafana)
+                                _time_ms=_time_ms)
     return (f'{asset_from}/{asset_to}', result) if return_pair else result
 
 
-def _asset_rate_lookup(asset_from, asset_to=None, date=None, _grafana=False):
+def _asset_rate_lookup(asset_from, asset_to=None, date=None, _time_ms=False):
     """
     Get asset rate for the specified date
 
@@ -882,13 +884,10 @@ def _asset_rate_lookup(asset_from, asset_to=None, date=None, _grafana=False):
 
     Function can be also called as e.g. asset_rate('EUR/USD')
     """
-    if _grafana:
-        if isinstance(date, int):
-            date = date / 1000
     if date is None:
         date = parse_date(return_timestamp=False)
     else:
-        date = parse_date(date, return_timestamp=False)
+        date = parse_date(date, return_timestamp=False, ms=_time_ms)
     asset_from, asset_to = _parse_asset_pair(asset_from, asset_to)
     if asset_from == asset_to:
         return 1
@@ -1803,7 +1802,7 @@ def account_statement(account,
                       tag=None,
                       pending=True,
                       datefmt=False,
-                      _grafana=False):
+                      _time_ms=False):
     """
     Args:
         account: account code
@@ -1815,20 +1814,15 @@ def account_statement(account,
     Returns:
         generator object
     """
-    if _grafana:
-        if isinstance(start, int):
-            start = start / 1000
-        if isinstance(end, int):
-            end = end / 1000
     acc_info = account_info(account)
     cond = 'transact.deleted is null and transact.service is null'
     d_field = 'd_created' if pending else 'd'
     if start:
-        dts = parse_date(start, return_timestamp=False)
+        dts = parse_date(start, return_timestamp=False, ms=_time_ms)
         cond += (' and ' if cond else '') + 'transact.{} >= \'{}\''.format(
             d_field, dts)
-    dte = parse_date(end, return_timestamp=False) if end else parse_date(
-        return_timestamp=False)
+    dte = parse_date(end, return_timestamp=False,
+                     ms=_time_ms) if end else parse_date(return_timestamp=False)
     cond += (' and ' if cond else '') + 'transact.{} <= \'{}\''.format(
         d_field, dte)
     if tag is not None:
@@ -1933,26 +1927,42 @@ def account_list(asset=None,
                  date=None,
                  base=None,
                  order_by=['tp', 'asset', 'account', 'balance'],
+                 group_by=None,
                  hide_empty=False,
-                 _grafana=False):
+                 _time_ms=False):
     """
     List accounts and their balances
 
     Args:
         asset: filter by asset
-        tp: filter by account type (or types)
+        tp: filter by account type (or types), value, list or values, separated
+            with |
         passive: list passive, active or all (if None) accounts
         code: filter by acocunt code (may contain '%' as a wildcards)
         date: get balances for the specified date
         base: convert account balances to base currency
         order_by: list ordering
+        group_by: 'asset' or 'type'
         hide_empty: hide accounts with zero balance, default is False
     """
+    if group_by is not None:
+        for acc in account_list_summary(asset=asset,
+                                        tp=tp,
+                                        passive=passive,
+                                        code=code,
+                                        date=date,
+                                        base=base,
+                                        order_by=order_by,
+                                        group_by=group_by,
+                                        hide_empty=hide_empty,
+                                        _time_ms=_time_ms,
+                                        _rsingle=True):
+            yield acc
+        return
     cond = "transact.deleted is null"
-    if _grafana:
-        if isinstance(date, int):
-            date = date / 1000
     if tp:
+        if '|' in tp:
+            tp = tp.split('|')
         tp = _safe_format(tp)
         if not isinstance(tp, (list, tuple)):
             if isinstance(tp, int):
@@ -1984,8 +1994,9 @@ def account_list(asset=None,
         cond += ')'
     else:
         cond += (' and ' if cond else '') + 'account.tp <= 1000'
-    dts = parse_date(date, return_timestamp=False) if date else parse_date(
-        return_timestamp=False)
+    dts = parse_date(date, return_timestamp=False,
+                     ms=_time_ms) if date else parse_date(
+                         return_timestamp=False)
     cond += (' and '
              if cond else '') + 'transact.d_created <= \'{}\''.format(dts)
     if code:
@@ -2070,7 +2081,9 @@ def account_list_summary(asset=None,
                          order_by=['tp', 'asset', 'account', 'balance'],
                          group_by=None,
                          hide_empty=False,
-                         base=None):
+                         base=None,
+                         _time_ms=False,
+                         _rsingle=False):
     """
     List accounts and their balances plus return a total sum
 
@@ -2101,9 +2114,11 @@ def account_list_summary(asset=None,
                      code=code,
                      date=date,
                      order_by=order_by,
-                     hide_empty=hide_empty))
+                     hide_empty=hide_empty,
+                     _time_ms=_time_ms))
     for a in accounts:
-        a['balance_bc'] = a['balance'] * asset_rate(a['asset'], base, date=date)
+        a['balance_bc'] = a['balance'] * asset_rate(
+            a['asset'], base, date=date, _time_ms=_time_ms)
     if group_by:
         res = []
         if group_by not in ('asset', 'tp', 'type'):
@@ -2120,7 +2135,9 @@ def account_list_summary(asset=None,
                     zip(dk, (k, sum([z['balance_bc'] for z in val
                                     ]), sum([w['balance'] for w in val]))))
                 res.append(r)
-        if group_by == 'asset':
+        if _rsingle:
+            return res
+        elif group_by == 'asset':
             return {
                 'assets':
                     res,
@@ -2128,9 +2145,9 @@ def account_list_summary(asset=None,
                     sum(
                         format_amount(d['balance'], d['asset'], d['passive'])
                         if d['asset'] == base else format_amount(
-                            d['balance'] *
-                            asset_rate(d['asset'], base, date=date), d['asset'],
-                            d['passive']) for d in accounts)
+                            d['balance'] * asset_rate(
+                                d['asset'], base, date=date, _time_ms=_time_ms),
+                            d['asset'], d['passive']) for d in accounts)
             }
         else:
             return {
@@ -2141,17 +2158,21 @@ def account_list_summary(asset=None,
                         format_amount(d['balance_bc'], base, d['passive'])
                         for d in accounts)
             }
-    return {
-        'accounts':
-            accounts,
-        'total':
-            sum(
-                format_amount(d['balance'], d['asset'], d['passive']
-                             ) if d['asset'] == base else format_amount(
-                                 d['balance'] *
-                                 asset_rate(d['asset'], base, date=date),
-                                 d['asset'], d['passive']) for d in accounts)
-    }
+    if _rsingle:
+        return accounts
+    else:
+        return {
+            'accounts':
+                accounts,
+            'total':
+                sum(
+                    format_amount(d['balance'], d['asset'], d['passive']
+                                 ) if d['asset'] == base else format_amount(
+                                     d['balance'] *
+                                     asset_rate(d['asset'], base, date=date),
+                                     d['asset'], d['passive'])
+                    for d in accounts)
+        }
 
 
 @core_method
@@ -2280,7 +2301,7 @@ def account_balance(account=None,
                     base=None,
                     date=None,
                     _natural=False,
-                    _grafana=False):
+                    _time_ms=False):
     """
     Get account balance
 
@@ -2296,12 +2317,10 @@ def account_balance(account=None,
         tp = [k for k in ACCOUNT_TYPE_IDS if ACCOUNT_TYPE_IDS[k] <= 1000]
     elif tp and '|' in tp:
         tp = [x.strip() for x in tp.split('|')]
-    if _grafana:
-        if isinstance(date, int):
-            date = date / 1000
     cond = "transact.deleted is null"
-    dts = parse_date(date, return_timestamp=False) if date else parse_date(
-        return_timestamp=False)
+    dts = parse_date(date, return_timestamp=False,
+                     ms=_time_ms) if date else parse_date(
+                         return_timestamp=False)
     cond += (' and '
              if cond else '') + 'transact.d_created <= \'{}\''.format(dts)
     balance = None
@@ -2336,7 +2355,8 @@ def account_balance(account=None,
                                         group_by='tp',
                                         date=date,
                                         base=base,
-                                        hide_empty=True)
+                                        hide_empty=True,
+                                        _time_ms=_time_ms)
         balance = accounts['total']
     return balance
 
@@ -2349,7 +2369,7 @@ def asset_rate_range(start,
                      step=1,
                      asset=None,
                      return_timestamp=False,
-                     _grafana=False):
+                     _time_ms=False):
     """
     Get list of asset rates for the specified time range
 
@@ -2370,7 +2390,7 @@ def asset_rate_range(start,
                            return_timestamp=return_timestamp,
                            fn=_asset_rate_lookup,
                            args=(asset_from, asset_to),
-                           _grafana=_grafana)
+                           _time_ms=_time_ms)
 
 
 @core_method
@@ -2381,7 +2401,7 @@ def account_balance_range(start,
                           step=1,
                           return_timestamp=False,
                           base=None,
-                          _grafana=False):
+                          _time_ms=False):
     """
     Get list of account balances for the specified time range
 
@@ -2418,7 +2438,7 @@ def account_balance_range(start,
                            kwargs={
                                **acc_info, 'base': base
                            },
-                           _grafana=_grafana)
+                           _time_ms=_time_ms)
 
 
 def _run_steps_func(start,
@@ -2428,17 +2448,12 @@ def _run_steps_func(start,
                     fn,
                     args=(),
                     kwargs={},
-                    _grafana=False):
-    if _grafana:
-        if isinstance(start, int):
-            start = start / 1000
-        if isinstance(end, int):
-            end = end / 1000
+                    _time_ms=False):
     times = []
     data = []
-    dt = parse_date(start, return_timestamp=False)
-    end_date = parse_date(
-        end, return_timestamp=False) if end else datetime.datetime.now()
+    dt = parse_date(start, return_timestamp=False, ms=_time_ms)
+    end_date = parse_date(end, return_timestamp=False,
+                          ms=_time_ms) if end else datetime.datetime.now()
     if isinstance(step, str) and step.endswith('a'):
         step = int(step[:-1])
         delta = (end_date - dt) / (step - 1) if step > 1 else None
